@@ -29,13 +29,13 @@
 #define ECS_RESPONSE_SIZE_LIMIT 10000
 #define ECS_CONNECT_TIMEOUT_DEFAULT_IN_SECONDS 2
 
+static void s_on_connection_manager_shutdown(void *user_data);
+
 struct aws_credentials_provider_ecs_impl {
     struct aws_http_connection_manager *connection_manager;
     struct aws_auth_http_system_vtable *function_table;
     struct aws_string *path_and_query;
     struct aws_string *auth_token;
-    struct aws_tls_ctx *ctx;
-    struct aws_tls_connection_options connection_options;
 };
 
 static struct aws_auth_http_system_vtable s_default_function_table = {
@@ -461,12 +461,18 @@ static void s_credentials_provider_ecs_destroy(struct aws_credentials_provider *
 
     aws_string_destroy(impl->path_and_query);
     aws_string_destroy(impl->auth_token);
-    aws_tls_connection_options_clean_up(&impl->connection_options);
+
     /* aws_http_connection_manager_release will eventually leads to call of s_on_connection_manager_shutdown,
      * which will do memory release for provider and impl. So We should be freeing impl
      * related memory first, then call aws_http_connection_manager_release.
      */
-    impl->function_table->aws_http_connection_manager_release(impl->connection_manager);
+    if (impl->connection_manager) {
+        impl->function_table->aws_http_connection_manager_release(impl->connection_manager);
+    } else {
+        /* If provider setup failed halfway through, connection_manager might not exist.
+         * In this case invoke shutdown completion callback directly to finish cleanup */
+        s_on_connection_manager_shutdown(provider);
+    }
 
     /* freeing the provider takes place in the shutdown callback below */
 }
@@ -478,10 +484,8 @@ static struct aws_credentials_provider_vtable s_aws_credentials_provider_ecs_vta
 
 static void s_on_connection_manager_shutdown(void *user_data) {
     struct aws_credentials_provider *provider = user_data;
-    struct aws_credentials_provider_ecs_impl *impl = provider->impl;
 
     aws_credentials_provider_invoke_shutdown_callback(provider);
-    aws_tls_ctx_release(impl->ctx);
     aws_mem_release(provider->allocator, provider);
 }
 
@@ -509,28 +513,12 @@ struct aws_credentials_provider *aws_credentials_provider_new_ecs(
 
     aws_credentials_provider_init_base(provider, allocator, &s_aws_credentials_provider_ecs_vtable, impl);
 
-    if (options->use_tls) {
-        AWS_LOGF_TRACE(
-            AWS_LS_AUTH_CREDENTIALS_PROVIDER,
-            "(id=%p): tls context not provided, initializing a new one",
-            (void *)provider);
-        struct aws_tls_ctx_options tls_options;
-        aws_tls_ctx_options_init_default_client(&tls_options, allocator);
-        impl->ctx = aws_tls_client_ctx_new(allocator, &tls_options);
-
-        if (!impl->ctx) {
-            AWS_LOGF_ERROR(
-                AWS_LS_AUTH_CREDENTIALS_PROVIDER,
-                "(id=%p): failed to create a tls context with error %s",
-                (void *)provider,
-                aws_error_debug_str(aws_last_error()));
-            aws_tls_ctx_options_clean_up(&tls_options);
-            goto on_error;
-        }
-
-        aws_tls_connection_options_init_from_ctx(&impl->connection_options, impl->ctx);
+    struct aws_tls_connection_options tls_connection_options;
+    AWS_ZERO_STRUCT(tls_connection_options);
+    if (options->tls_ctx) {
+        aws_tls_connection_options_init_from_ctx(&tls_connection_options, options->tls_ctx);
         struct aws_byte_cursor host = options->host;
-        if (aws_tls_connection_options_set_server_name(&impl->connection_options, allocator, &host)) {
+        if (aws_tls_connection_options_set_server_name(&tls_connection_options, allocator, &host)) {
             AWS_LOGF_ERROR(
                 AWS_LS_AUTH_CREDENTIALS_PROVIDER,
                 "(id=%p): failed to create a tls connection options with error %s",
@@ -553,11 +541,11 @@ struct aws_credentials_provider *aws_credentials_provider_new_ecs(
     manager_options.initial_window_size = ECS_RESPONSE_SIZE_LIMIT;
     manager_options.socket_options = &socket_options;
     manager_options.host = options->host;
-    manager_options.port = options->use_tls ? 443 : 80;
+    manager_options.port = options->tls_ctx ? 443 : 80;
     manager_options.max_connections = 2;
     manager_options.shutdown_complete_callback = s_on_connection_manager_shutdown;
     manager_options.shutdown_complete_user_data = provider;
-    manager_options.tls_connection_options = options->use_tls ? &(impl->connection_options) : NULL;
+    manager_options.tls_connection_options = options->tls_ctx ? &tls_connection_options : NULL;
 
     impl->function_table = options->function_table;
     if (impl->function_table == NULL) {
@@ -581,10 +569,13 @@ struct aws_credentials_provider *aws_credentials_provider_new_ecs(
     }
     provider->shutdown_options = options->shutdown_options;
 
+    aws_tls_connection_options_clean_up(&tls_connection_options);
+
     return provider;
 
 on_error:
 
+    aws_tls_connection_options_clean_up(&tls_connection_options);
     aws_credentials_provider_destroy(provider);
 
     return NULL;
