@@ -160,15 +160,26 @@ struct posix_socket_connect_args {
 
 struct posix_socket {
     struct aws_linked_list write_queue;
+    struct aws_linked_list written_queue;
+    struct aws_task written_task;
     struct posix_socket_connect_args *connect_args;
-    bool write_in_progress;
+    /* Note that only the posix_socket impl part is refcounted.
+     * The public aws_socket can be a stack variable and cleaned up synchronously
+     * (by blocking until the event-loop cleans up the impl part).
+     * In hindsight, aws_socket should have been heap-allocated and refcounted, but alas */
+    struct aws_ref_count internal_refcount;
+    struct aws_allocator *allocator;
+    bool written_task_scheduled;
     bool currently_subscribed;
     bool continue_accept;
-    bool currently_in_event;
-    bool clean_yourself_up;
     bool *close_happened;
 };
 
+static void s_socket_destroy_impl(void *user_data) {
+    struct posix_socket *socket_impl = user_data;
+    aws_mem_release(socket_impl->allocator, socket_impl);
+}
+
 static int s_socket_init(
     struct aws_socket *socket,
     struct aws_allocator *alloc,
@@ -204,11 +215,11 @@ static int s_socket_init(
     }
 
     aws_linked_list_init(&posix_socket->write_queue);
-    posix_socket->write_in_progress = false;
+    aws_linked_list_init(&posix_socket->written_queue);
     posix_socket->currently_subscribed = false;
     posix_socket->continue_accept = false;
-    posix_socket->currently_in_event = false;
-    posix_socket->clean_yourself_up = false;
+    aws_ref_count_init(&posix_socket->internal_refcount, posix_socket, s_socket_destroy_impl);
+    posix_socket->allocator = alloc;
     posix_socket->connect_args = NULL;
     posix_socket->close_happened = NULL;
     socket->impl = posix_socket;
@@ -225,22 +236,22 @@ void aws_socket_clean_up(struct aws_socket *socket) {
         /* protect from double clean */
         return;
     }
+
+    int fd_for_logging = socket->io_handle.data.fd; /* socket's fd gets reset before final log */
+    (void)fd_for_logging;
+
     if (aws_socket_is_open(socket)) {
-        AWS_LOGF_DEBUG(
-            AWS_LS_IO_SOCKET, "id=%p fd=%d: is still open, closing...", (void *)socket, socket->io_handle.data.fd);
+        AWS_LOGF_DEBUG(AWS_LS_IO_SOCKET, "id=%p fd=%d: is still open, closing...", (void *)socket, fd_for_logging);
         aws_socket_close(socket);
     }
     struct posix_socket *socket_impl = socket->impl;
 
-    if (!socket_impl->currently_in_event) {
-        aws_mem_release(socket->allocator, socket->impl);
-    } else {
+    if (aws_ref_count_release(&socket_impl->internal_refcount) != 0) {
         AWS_LOGF_DEBUG(
             AWS_LS_IO_SOCKET,
             "id=%p fd=%d: is still pending io letting it dangle and cleaning up later.",
             (void *)socket,
-            socket->io_handle.data.fd);
-        socket_impl->clean_yourself_up = true;
+            fd_for_logging);
     }
 
     AWS_ZERO_STRUCT(*socket);
@@ -1083,12 +1094,14 @@ int aws_socket_stop_accept(struct aws_socket *socket) {
         AWS_LS_IO_SOCKET, "id=%p fd=%d: stopping accepting new connections", (void *)socket, socket->io_handle.data.fd);
 
     if (!aws_event_loop_thread_is_callers_thread(socket->event_loop)) {
-        struct stop_accept_args args = {.mutex = AWS_MUTEX_INIT,
-                                        .condition_variable = AWS_CONDITION_VARIABLE_INIT,
-                                        .invoked = false,
-                                        .socket = socket,
-                                        .ret_code = AWS_OP_SUCCESS,
-                                        .task = {.fn = s_stop_accept_task}};
+        struct stop_accept_args args = {
+            .mutex = AWS_MUTEX_INIT,
+            .condition_variable = AWS_CONDITION_VARIABLE_INIT,
+            .invoked = false,
+            .socket = socket,
+            .ret_code = AWS_OP_SUCCESS,
+            .task = {.fn = s_stop_accept_task},
+        };
         AWS_LOGF_INFO(
             AWS_LS_IO_SOCKET,
             "id=%p fd=%d: stopping accepting new connections from a different thread than "
@@ -1229,6 +1242,7 @@ struct write_request {
     void *write_user_data;
     struct aws_linked_list_node node;
     size_t original_buffer_len;
+    int error_code;
 };
 
 struct posix_socket_close_args {
@@ -1264,6 +1278,7 @@ static void s_close_task(struct aws_task *task, void *arg, enum aws_task_status
 int aws_socket_close(struct aws_socket *socket) {
     struct posix_socket *socket_impl = socket->impl;
     AWS_LOGF_DEBUG(AWS_LS_IO_SOCKET, "id=%p fd=%d: closing", (void *)socket, socket->io_handle.data.fd);
+    struct aws_event_loop *event_loop = socket->event_loop;
     if (socket->event_loop) {
         /* don't freak out on me, this almost never happens, and never occurs inside a channel
          * it only gets hit from a listening socket shutting down or from a unit test. */
@@ -1292,12 +1307,14 @@ int aws_socket_close(struct aws_socket *socket) {
                 .arg = &args,
             };
 
+            int fd_for_logging = socket->io_handle.data.fd; /* socket's fd gets reset before final log */
+            (void)fd_for_logging;
+
             aws_mutex_lock(&args.mutex);
             aws_event_loop_schedule_task_now(socket->event_loop, &close_task);
             aws_condition_variable_wait_pred(&args.condition_variable, &args.mutex, s_close_predicate, &args);
             aws_mutex_unlock(&args.mutex);
-            AWS_LOGF_INFO(
-                AWS_LS_IO_SOCKET, "id=%p fd=%d: close task completed.", (void *)socket, socket->io_handle.data.fd);
+            AWS_LOGF_INFO(AWS_LS_IO_SOCKET, "id=%p fd=%d: close task completed.", (void *)socket, fd_for_logging);
             if (args.ret_code) {
                 return aws_raise_error(args.ret_code);
             }
@@ -1334,14 +1351,25 @@ int aws_socket_close(struct aws_socket *socket) {
         socket->io_handle.data.fd = -1;
         socket->state = CLOSED;
 
-        /* after close, just go ahead and clear out the pending writes queue
-         * and tell the user they were cancelled. */
+        /* ensure callbacks for pending writes fire (in order) before this close function returns */
+
+        if (socket_impl->written_task_scheduled) {
+            aws_event_loop_cancel_task(event_loop, &socket_impl->written_task);
+        }
+
+        while (!aws_linked_list_empty(&socket_impl->written_queue)) {
+            struct aws_linked_list_node *node = aws_linked_list_pop_front(&socket_impl->written_queue);
+            struct write_request *write_request = AWS_CONTAINER_OF(node, struct write_request, node);
+            size_t bytes_written = write_request->original_buffer_len - write_request->cursor_cpy.len;
+            write_request->written_fn(socket, write_request->error_code, bytes_written, write_request->write_user_data);
+            aws_mem_release(socket->allocator, write_request);
+        }
+
         while (!aws_linked_list_empty(&socket_impl->write_queue)) {
             struct aws_linked_list_node *node = aws_linked_list_pop_front(&socket_impl->write_queue);
             struct write_request *write_request = AWS_CONTAINER_OF(node, struct write_request, node);
-
-            write_request->written_fn(
-                socket, AWS_IO_SOCKET_CLOSED, write_request->original_buffer_len, write_request->write_user_data);
+            size_t bytes_written = write_request->original_buffer_len - write_request->cursor_cpy.len;
+            write_request->written_fn(socket, AWS_IO_SOCKET_CLOSED, bytes_written, write_request->write_user_data);
             aws_mem_release(socket->allocator, write_request);
         }
     }
@@ -1367,21 +1395,53 @@ int aws_socket_shutdown_dir(struct aws_socket *socket, enum aws_channel_directio
     return AWS_OP_SUCCESS;
 }
 
+static void s_written_task(struct aws_task *task, void *arg, enum aws_task_status status) {
+    (void)task;
+    (void)status;
+
+    struct aws_socket *socket = arg;
+    struct posix_socket *socket_impl = socket->impl;
+
+    socket_impl->written_task_scheduled = false;
+
+    /* this is to handle a race condition when a callback kicks off a cleanup, or the user decides
+     * to close the socket based on something they read (SSL validation failed for example).
+     * if clean_up happens when internal_refcount > 0, socket_impl is kept dangling */
+    aws_ref_count_acquire(&socket_impl->internal_refcount);
+
+    /* Notes about weird loop:
+     * 1) Only process the initial contents of queue when this task is run,
+     *    ignoring any writes queued during delivery.
+     *    If we simply looped until the queue was empty, we could get into a
+     *    synchronous loop of completing and writing and completing and writing...
+     *    and it would be tough for multiple sockets to share an event-loop fairly.
+     * 2) Check if queue is empty with each iteration.
+     *    If user calls close() from the callback, close() will process all
+     *    nodes in the written_queue, and the queue will be empty when the
+     *    callstack gets back to here. */
+    if (!aws_linked_list_empty(&socket_impl->written_queue)) {
+        struct aws_linked_list_node *stop_after = aws_linked_list_back(&socket_impl->written_queue);
+        do {
+            struct aws_linked_list_node *node = aws_linked_list_pop_front(&socket_impl->written_queue);
+            struct write_request *write_request = AWS_CONTAINER_OF(node, struct write_request, node);
+            size_t bytes_written = write_request->original_buffer_len - write_request->cursor_cpy.len;
+            write_request->written_fn(socket, write_request->error_code, bytes_written, write_request->write_user_data);
+            aws_mem_release(socket_impl->allocator, write_request);
+            if (node == stop_after) {
+                break;
+            }
+        } while (!aws_linked_list_empty(&socket_impl->written_queue));
+    }
+
+    aws_ref_count_release(&socket_impl->internal_refcount);
+}
+
 /* this gets called in two scenarios.
  * 1st scenario, someone called aws_socket_write() and we want to try writing now, so an error can be returned
  * immediately if something bad has happened to the socket. In this case, `parent_request` is set.
  * 2nd scenario, the event loop notified us that the socket went writable. In this case `parent_request` is NULL */
 static int s_process_write_requests(struct aws_socket *socket, struct write_request *parent_request) {
     struct posix_socket *socket_impl = socket->impl;
-    struct aws_allocator *allocator = socket->allocator;
-
-    AWS_LOGF_TRACE(
-        AWS_LS_IO_SOCKET, "id=%p fd=%d: processing write requests.", (void *)socket, socket->io_handle.data.fd);
-
-    /* there's a potential deadlock where we notify the user that we wrote some data, the user
-     * says, "cool, now I can write more and then immediately calls aws_socket_write(). We need to make sure
-     * that we don't allow reentrancy in that case. */
-    socket_impl->write_in_progress = true;
 
     if (parent_request) {
         AWS_LOGF_TRACE(
@@ -1389,7 +1449,6 @@ static int s_process_write_requests(struct aws_socket *socket, struct write_requ
             "id=%p fd=%d: processing write requests, called from aws_socket_write",
             (void *)socket,
             socket->io_handle.data.fd);
-        socket_impl->currently_in_event = true;
     } else {
         AWS_LOGF_TRACE(
             AWS_LS_IO_SOCKET,
@@ -1401,6 +1460,7 @@ static int s_process_write_requests(struct aws_socket *socket, struct write_requ
     bool purge = false;
     int aws_error = AWS_OP_SUCCESS;
     bool parent_request_failed = false;
+    bool pushed_to_written_queue = false;
 
     /* if a close call happens in the middle, this queue will have been cleaned out from under us. */
     while (!aws_linked_list_empty(&socket_impl->write_queue)) {
@@ -1472,9 +1532,9 @@ static int s_process_write_requests(struct aws_socket *socket, struct write_requ
                 AWS_LS_IO_SOCKET, "id=%p fd=%d: write request completed", (void *)socket, socket->io_handle.data.fd);
 
             aws_linked_list_remove(node);
-            write_request->written_fn(
-                socket, AWS_OP_SUCCESS, write_request->original_buffer_len, write_request->write_user_data);
-            aws_mem_release(allocator, write_request);
+            write_request->error_code = AWS_ERROR_SUCCESS;
+            aws_linked_list_push_back(&socket_impl->written_queue, node);
+            pushed_to_written_queue = true;
         }
     }
 
@@ -1487,22 +1547,19 @@ static int s_process_write_requests(struct aws_socket *socket, struct write_requ
              * as the user will be able to rely on the return value from aws_socket_write() */
             if (write_request == parent_request) {
                 parent_request_failed = true;
+                aws_mem_release(socket->allocator, write_request);
             } else {
-                write_request->written_fn(socket, aws_error, 0, write_request->write_user_data);
+                write_request->error_code = aws_error;
+                aws_linked_list_push_back(&socket_impl->written_queue, node);
+                pushed_to_written_queue = true;
             }
-
-            aws_mem_release(socket->allocator, write_request);
         }
     }
 
-    socket_impl->write_in_progress = false;
-
-    if (parent_request) {
-        socket_impl->currently_in_event = false;
-    }
-
-    if (socket_impl->clean_yourself_up) {
-        aws_mem_release(allocator, socket_impl);
+    if (pushed_to_written_queue && !socket_impl->written_task_scheduled) {
+        socket_impl->written_task_scheduled = true;
+        aws_task_init(&socket_impl->written_task, s_written_task, socket, "socket_written_task");
+        aws_event_loop_schedule_task_now(socket->event_loop, &socket_impl->written_task);
     }
 
     /* Only report error if aws_socket_write() invoked this function and its write_request failed */
@@ -1521,15 +1578,14 @@ static void s_on_socket_io_event(
     void *user_data) {
     (void)event_loop;
     (void)handle;
-    /* this is to handle a race condition when an error kicks off a cleanup, or the user decides
-     * to close the socket based on something they read (SSL validation failed for example).
-     * if clean_up happens when currently_in_event is true, socket_impl is kept dangling but currently
-     * subscribed is set to false. */
     struct aws_socket *socket = user_data;
     struct posix_socket *socket_impl = socket->impl;
-    struct aws_allocator *allocator = socket->allocator;
 
-    socket_impl->currently_in_event = true;
+    /* this is to handle a race condition when an error kicks off a cleanup, or the user decides
+     * to close the socket based on something they read (SSL validation failed for example).
+     * if clean_up happens when internal_refcount > 0, socket_impl is kept dangling but currently
+     * subscribed is set to false. */
+    aws_ref_count_acquire(&socket_impl->internal_refcount);
 
     if (events & AWS_IO_EVENT_TYPE_REMOTE_HANG_UP || events & AWS_IO_EVENT_TYPE_CLOSED) {
         aws_raise_error(AWS_IO_SOCKET_CLOSED);
@@ -1565,11 +1621,7 @@ static void s_on_socket_io_event(
     }
 
 end_check:
-    socket_impl->currently_in_event = false;
-
-    if (socket_impl->clean_yourself_up) {
-        aws_mem_release(allocator, socket_impl);
-    }
+    aws_ref_count_release(&socket_impl->internal_refcount);
 }
 
 int aws_socket_assign_to_event_loop(struct aws_socket *socket, struct aws_event_loop *event_loop) {
@@ -1749,12 +1801,7 @@ int aws_socket_write(
     write_request->cursor_cpy = *cursor;
     aws_linked_list_push_back(&socket_impl->write_queue, &write_request->node);
 
-    /* avoid reentrancy when a user calls write after receiving their completion callback. */
-    if (!socket_impl->write_in_progress) {
-        return s_process_write_requests(socket, write_request);
-    }
-
-    return AWS_OP_SUCCESS;
+    return s_process_write_requests(socket, write_request);
 }
 
 int aws_socket_get_error(struct aws_socket *socket) {
