@@ -114,25 +114,18 @@ static int s_test_s3_client_acquire_connection_fail(struct aws_allocator *alloca
     return 0;
 }
 
-static int s_s3_fail_first_prepare_request(struct aws_s3_meta_request *meta_request, struct aws_s3_request *request) {
+struct s3_fail_prepare_test_data {
+    uint32_t num_requests_being_prepared_is_correct : 1;
+};
 
+static int s_s3_fail_prepare_request(struct aws_s3_meta_request *meta_request, struct aws_s3_request *request) {
+    (void)meta_request;
+    (void)request;
+    AWS_ASSERT(meta_request != NULL);
     AWS_ASSERT(request != NULL);
 
-    struct aws_s3_client *client = meta_request->client;
-    AWS_ASSERT(client != NULL);
-
-    struct aws_s3_tester *tester = client->shutdown_callback_user_data;
-    AWS_ASSERT(tester != NULL);
-
-    if (aws_s3_tester_inc_counter1(tester) == 1) {
-        aws_raise_error(AWS_ERROR_UNKNOWN);
-        return AWS_OP_ERR;
-    }
-
-    struct aws_s3_meta_request_vtable *original_meta_request_vtable =
-        aws_s3_tester_get_meta_request_vtable_patch(tester, 0)->original_vtable;
-
-    return original_meta_request_vtable->prepare_request(meta_request, request);
+    aws_raise_error(AWS_ERROR_UNKNOWN);
+    return AWS_OP_ERR;
 }
 
 static struct aws_s3_meta_request *s_meta_request_factory_patch_prepare_request(
@@ -151,11 +144,28 @@ static struct aws_s3_meta_request *s_meta_request_factory_patch_prepare_request(
 
     struct aws_s3_meta_request_vtable *patched_meta_request_vtable =
         aws_s3_tester_patch_meta_request_vtable(tester, meta_request, NULL);
-    patched_meta_request_vtable->prepare_request = s_s3_fail_first_prepare_request;
+    patched_meta_request_vtable->prepare_request = s_s3_fail_prepare_request;
 
     return meta_request;
 }
 
+static void s_s3_fail_prepare_finish_destroy(struct aws_s3_client *client) {
+    AWS_ASSERT(client);
+
+    struct aws_s3_tester *tester = client->shutdown_callback_user_data;
+    AWS_ASSERT(tester != NULL);
+
+    struct s3_fail_prepare_test_data *test_data = tester->user_data;
+    AWS_ASSERT(test_data != NULL);
+
+    test_data->num_requests_being_prepared_is_correct = client->threaded_data.num_requests_being_prepared == 0;
+
+    struct aws_s3_client_vtable *original_client_vtable =
+        aws_s3_tester_get_client_vtable_patch(tester, 0)->original_vtable;
+
+    original_client_vtable->finish_destroy(client);
+}
+
 /* Test recovery when prepare request fails. */
 AWS_TEST_CASE(test_s3_meta_request_fail_prepare_request, s_test_s3_meta_request_fail_prepare_request)
 static int s_test_s3_meta_request_fail_prepare_request(struct aws_allocator *allocator, void *ctx) {
@@ -164,6 +174,10 @@ static int s_test_s3_meta_request_fail_prepare_request(struct aws_allocator *all
     struct aws_s3_tester tester;
     ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester));
 
+    struct s3_fail_prepare_test_data test_data;
+    AWS_ZERO_STRUCT(test_data);
+    tester.user_data = &test_data;
+
     struct aws_s3_client_config client_config;
     AWS_ZERO_STRUCT(client_config);
 
@@ -173,14 +187,19 @@ static int s_test_s3_meta_request_fail_prepare_request(struct aws_allocator *all
     struct aws_s3_client *client = aws_s3_client_new(allocator, &client_config);
     struct aws_s3_client_vtable *patched_client_vtable = aws_s3_tester_patch_client_vtable(&tester, client, NULL);
     patched_client_vtable->meta_request_factory = s_meta_request_factory_patch_prepare_request;
+    patched_client_vtable->finish_destroy = s_s3_fail_prepare_finish_destroy;
 
     ASSERT_SUCCESS(aws_s3_tester_send_get_object_meta_request(&tester, client, g_s3_path_get_object_test_1MB, 0, NULL));
 
+    aws_s3_tester_wait_for_counters(&tester);
+
     aws_s3_client_release(client);
     client = NULL;
 
     aws_s3_tester_clean_up(&tester);
 
+    ASSERT_TRUE(test_data.num_requests_being_prepared_is_correct);
+
     return 0;
 }
 
