@@ -195,7 +195,7 @@ int cache_retrieve_callback(struct s2n_connection *conn, void *ctx, const void *
     *value_size = cache[idx].value_len;
     memcpy(value, cache[idx].value, cache[idx].value_len);
 
-    for (int i = 0; i < key_size; i++) {
+    for (uint64_t i = 0; i < key_size; i++) {
         printf("%02x", ((const uint8_t *)key)[i]);
     }
     printf("\n");
@@ -372,8 +372,16 @@ int handle_connection(int fd, struct s2n_config *config, struct conn_settings se
         echo(conn, fd);
     }
 
+    /* The following call can block on receiving a close_notify if we initiate the shutdown or if the */
+    /* peer fails to send a close_notify. */
+    /* TODO: However, we should expect to receive a close_notify from the peer and shutdown gracefully. */
+    /* Please see tracking issue for more detail: https://github.com/aws/s2n-tls/issues/2692 */
     s2n_blocked_status blocked;
-    s2n_shutdown(conn, &blocked);
+    int shutdown_rc = s2n_shutdown(conn, &blocked);
+    if (shutdown_rc == -1 && blocked != S2N_BLOCKED_ON_READ) {
+        fprintf(stderr, "Unexpected error during shutdown: '%s'\n", s2n_strerror(s2n_errno, "NULL"));
+        exit(1);
+    }
 
     GUARD_RETURN(s2n_connection_wipe(conn), "Error wiping connection");
 
