@@ -24,6 +24,7 @@
 #include <strings.h>
 
 #include "crypto/s2n_certificate.h"
+#include <crypto/s2n_openssl_x509.h>
 #include "utils/s2n_array.h"
 #include "utils/s2n_safety.h"
 #include "utils/s2n_mem.h"
@@ -402,6 +403,25 @@ int s2n_cert_chain_and_key_free(struct s2n_cert_chain_and_key *cert_and_key)
     return 0;
 }
 
+int s2n_cert_chain_free(struct s2n_cert_chain *cert_chain)
+{
+    /* Walk the chain and free the certs/nodes allocated prior to failure */
+    if (cert_chain) {
+        struct s2n_cert *node = cert_chain->head;
+        while (node) {
+            /* Free the cert */
+            POSIX_GUARD(s2n_free(&node->raw));
+            /* update head so it won't point to freed memory */
+            cert_chain->head = node->next;
+            /* Free the node */
+            POSIX_GUARD(s2n_free_object((uint8_t **)&node, sizeof(struct s2n_cert)));
+            node = cert_chain->head;
+        }
+    }
+
+    return S2N_SUCCESS;
+}
+
 int s2n_send_cert_chain(struct s2n_connection *conn, struct s2n_stuffer *out, struct s2n_cert_chain_and_key *chain_and_key)
 {
     POSIX_ENSURE_REF(conn);
@@ -532,3 +552,246 @@ s2n_cert_private_key *s2n_cert_chain_and_key_get_private_key(struct s2n_cert_cha
     PTR_ENSURE_REF(chain_and_key);
     return chain_and_key->private_key;
 }
+
+int s2n_cert_chain_get_length(const struct s2n_cert_chain_and_key *chain_and_key, uint32_t *cert_length)
+{
+    POSIX_ENSURE_REF(chain_and_key);
+    POSIX_ENSURE_REF(cert_length);
+
+    struct s2n_cert *head_cert = chain_and_key->cert_chain->head;
+    POSIX_ENSURE_REF(head_cert);
+    *cert_length = 1;
+    struct s2n_cert *next_cert = head_cert->next;
+    while (next_cert != NULL) {
+        *cert_length += 1;
+        next_cert = next_cert->next;
+    }
+
+    return S2N_SUCCESS;
+}
+
+int s2n_cert_chain_get_cert(const struct s2n_cert_chain_and_key *chain_and_key, struct s2n_cert **out_cert,
+                                 const uint32_t cert_idx)
+{
+    POSIX_ENSURE_REF(chain_and_key);
+    POSIX_ENSURE_REF(out_cert);
+
+    struct s2n_cert *cur_cert = chain_and_key->cert_chain->head;
+    POSIX_ENSURE_REF(cur_cert);
+    uint32_t counter = 0;
+
+    struct s2n_cert *next_cert = cur_cert->next;
+
+    while ((next_cert != NULL) && (counter < cert_idx)) {
+        cur_cert  = next_cert;
+        next_cert = next_cert->next;
+        counter++;
+    }
+
+    POSIX_ENSURE(counter == cert_idx, S2N_ERR_NO_CERT_FOUND);
+    POSIX_ENSURE(cur_cert != NULL, S2N_ERR_NO_CERT_FOUND);
+    *out_cert = cur_cert;
+
+    return S2N_SUCCESS;
+}
+
+int s2n_cert_get_der(const struct s2n_cert *cert, const uint8_t **out_cert_der, uint32_t *cert_length)
+{
+    POSIX_ENSURE_REF(cert);
+    POSIX_ENSURE_REF(out_cert_der);
+    POSIX_ENSURE_REF(cert_length);
+
+    *cert_length = cert->raw.size;
+    *out_cert_der = cert->raw.data;
+
+    return S2N_SUCCESS;
+}
+
+static int s2n_asn1_obj_free(ASN1_OBJECT ** data)
+{
+    if (*data != NULL) {
+         ASN1_OBJECT_free(*data);
+    }
+    return S2N_SUCCESS;
+}
+
+static int s2n_asn1_string_free(ASN1_STRING** data)
+{
+    if (*data != NULL) {
+        ASN1_STRING_free(*data);
+    }
+    return S2N_SUCCESS;
+}
+
+static int s2n_utf8_string_from_extension_data(const uint8_t *extension_data, uint32_t extension_len, uint8_t *out_data, uint32_t *out_len)
+{
+    DEFER_CLEANUP(ASN1_STRING *asn1_str = NULL, s2n_asn1_string_free);
+    /* Note that d2i_ASN1_UTF8STRING increments *der_in to the byte following the parsed data.
+     * Using a temporary variable is mandatory to prevent memory free-ing errors.
+     * Ref to the warning section here for more information:
+     * https://www.openssl.org/docs/man1.1.0/man3/d2i_ASN1_UTF8STRING.html.
+     */
+    const uint8_t *asn1_str_data = extension_data;
+    asn1_str = d2i_ASN1_UTF8STRING(NULL, (const unsigned char **)(void *)&asn1_str_data, extension_len);
+    POSIX_ENSURE(asn1_str != NULL, S2N_ERR_INVALID_X509_EXTENSION_TYPE);
+    /* ASN1_STRING_type() returns the type of `asn1_str`, using standard constants such as V_ASN1_OCTET_STRING.
+     * Ref: https://www.openssl.org/docs/man1.1.0/man3/ASN1_STRING_type.html. 
+     */
+    int type = ASN1_STRING_type(asn1_str);
+    POSIX_ENSURE(type == V_ASN1_UTF8STRING, S2N_ERR_INVALID_X509_EXTENSION_TYPE);
+
+    int len = ASN1_STRING_length(asn1_str);
+    if (out_data != NULL) {
+        POSIX_ENSURE(*out_len >= len, S2N_ERR_INSUFFICIENT_MEM_SIZE);
+        /* ASN1_STRING_data() returns an internal pointer to the data. 
+        * Since this is an internal pointer it should not be freed or modified in any way.
+        * Ref: https://www.openssl.org/docs/man1.0.2/man3/ASN1_STRING_data.html.
+        */ 
+        unsigned char *internal_data = ASN1_STRING_data(asn1_str);
+        POSIX_ENSURE_REF(internal_data);
+        POSIX_CHECKED_MEMCPY(out_data, internal_data, len);
+    }
+    *out_len = len;
+    return S2N_SUCCESS; 
+}
+
+int s2n_cert_get_utf8_string_from_extension_data_length(const uint8_t *extension_data, uint32_t extension_len, uint32_t *utf8_str_len)
+{
+    POSIX_ENSURE_REF(extension_data);
+    POSIX_ENSURE_GT(extension_len, 0);
+    POSIX_ENSURE_REF(utf8_str_len);
+
+    POSIX_GUARD(s2n_utf8_string_from_extension_data(extension_data, extension_len, NULL, utf8_str_len));
+
+    return S2N_SUCCESS;
+}
+
+int s2n_cert_get_utf8_string_from_extension_data(const uint8_t *extension_data, uint32_t extension_len, uint8_t *out_data, uint32_t *out_len)
+{
+    POSIX_ENSURE_REF(extension_data);
+    POSIX_ENSURE_GT(extension_len, 0);
+    POSIX_ENSURE_REF(out_data);
+    POSIX_ENSURE_REF(out_len);
+
+    POSIX_GUARD(s2n_utf8_string_from_extension_data(extension_data, extension_len, out_data, out_len));
+
+    return S2N_SUCCESS;
+}
+
+static int s2n_parse_x509_extension(struct s2n_cert *cert, const uint8_t *oid,
+                                      uint8_t *ext_value, uint32_t *ext_value_len, bool *critical)
+{
+    POSIX_ENSURE_REF(cert->raw.data);
+    /* Obtain the openssl x509 cert from the ASN1 DER certificate input. 
+     * Note that d2i_X509 increments *der_in to the byte following the parsed data.
+     * Using a temporary variable is mandatory to prevent memory free-ing errors.
+     * Ref to the warning section here for more information:
+     * https://www.openssl.org/docs/man1.1.0/man3/d2i_X509.html.
+     */
+    uint8_t *der_in = cert->raw.data;
+    DEFER_CLEANUP(X509 *x509_cert = d2i_X509(NULL, (const unsigned char **)(void *)&der_in, cert->raw.size),
+                  X509_free_pointer);
+    POSIX_ENSURE_REF(x509_cert);
+
+    /* Retrieve the number of x509 extensions present in the certificate 
+     * X509_get_ext_count returns the number of extensions in the x509 certificate. 
+     * Ref: https://www.openssl.org/docs/man1.1.0/man3/X509_get_ext_count.html.
+     */
+    int ext_count = X509_get_ext_count(x509_cert);
+    POSIX_ENSURE_GT(ext_count, 0);
+
+    /* OBJ_txt2obj() converts the input text string into an ASN1_OBJECT structure.
+     * If no_name is 0 then long names and short names will be interpreted as well as numerical forms.
+     * If no_name is 1 only the numerical form is acceptable. 
+     * Ref: https://www.openssl.org/docs/man1.1.0/man3/OBJ_txt2obj.html.
+     */
+    DEFER_CLEANUP(ASN1_OBJECT *asn1_obj_in = OBJ_txt2obj((const char *)oid, 0), s2n_asn1_obj_free);
+    POSIX_ENSURE_REF(asn1_obj_in);
+
+    for (size_t loc = 0; loc < ext_count; loc++) {
+        ASN1_OCTET_STRING *asn1_str = NULL;
+        bool match_found = false; 
+
+        /* Retrieve the x509 extension at location loc.
+         * X509_get_ext() retrieves extension loc from x.
+         * The index loc can take any value from 0 to X509_get_ext_count(x) - 1.
+         * The returned extension is an internal pointer which must not be freed up by the application.
+         * Ref: https://www.openssl.org/docs/man1.1.0/man3/X509_get_ext.html.
+         */
+        X509_EXTENSION *x509_ext = X509_get_ext(x509_cert, loc);
+        POSIX_ENSURE_REF(x509_ext);
+
+        /* Retrieve the extension object/OID/extnId. 
+         * X509_EXTENSION_get_object() returns the extension type of `x509_ext` as an ASN1_OBJECT pointer. 
+         * The returned pointer is an internal value which must not be freed up. 
+         * Ref: https://www.openssl.org/docs/man1.1.0/man3/X509_EXTENSION_get_object.html.
+         */
+        ASN1_OBJECT *asn1_obj = X509_EXTENSION_get_object(x509_ext);
+        POSIX_ENSURE_REF(asn1_obj);
+
+        /* OBJ_cmp() compares two ASN1_OBJECT objects. If the two are identical 0 is returned.
+         * Ref: https://www.openssl.org/docs/man1.1.0/man3/OBJ_cmp.html.
+         */
+        match_found = (0 == OBJ_cmp(asn1_obj_in, asn1_obj));
+
+        /* If match found, retrieve the corresponding OID value for the x509 extension */
+        if (match_found) {
+            /* X509_EXTENSION_get_data() returns the data of extension `x509_ext`. 
+             * The returned pointer is an internal value which must not be freed up.
+             * Ref: https://www.openssl.org/docs/man1.1.0/man3/X509_EXTENSION_get_data.html.
+             */
+            asn1_str = X509_EXTENSION_get_data(x509_ext);
+            /* ASN1_STRING_length() returns the length of the content of `asn1_str`.
+            * Ref: https://www.openssl.org/docs/man1.1.0/man3/ASN1_STRING_length.html.
+            */    
+            int len = ASN1_STRING_length(asn1_str);
+            if (ext_value != NULL) {
+                POSIX_ENSURE(*ext_value_len >= len, S2N_ERR_INSUFFICIENT_MEM_SIZE);
+                /* ASN1_STRING_data() returns an internal pointer to the data. 
+                * Since this is an internal pointer it should not be freed or modified in any way.
+                * Ref: https://www.openssl.org/docs/man1.0.2/man3/ASN1_STRING_data.html.
+                */
+                unsigned char *internal_data = ASN1_STRING_data(asn1_str);
+                POSIX_ENSURE_REF(internal_data);
+                POSIX_CHECKED_MEMCPY(ext_value, internal_data, len);
+            }
+            if (critical != NULL) {
+               /* Retrieve the x509 extension's critical value.
+                * X509_EXTENSION_get_critical() returns the criticality of extension `x509_ext`,
+                * it returns 1 for critical and 0 for non-critical.
+                * Ref: https://www.openssl.org/docs/man1.1.0/man3/X509_EXTENSION_get_critical.html.
+                */
+                *critical = X509_EXTENSION_get_critical(x509_ext);
+            }
+            *ext_value_len = len;
+            return S2N_SUCCESS;
+        }
+    }
+
+    POSIX_BAIL(S2N_ERR_X509_EXTENSION_VALUE_NOT_FOUND);
+}
+
+int s2n_cert_get_x509_extension_value_length(struct s2n_cert *cert, const uint8_t *oid, uint32_t *ext_value_len)
+{
+    POSIX_ENSURE_REF(cert);
+    POSIX_ENSURE_REF(oid);
+    POSIX_ENSURE_REF(ext_value_len);
+
+    POSIX_GUARD(s2n_parse_x509_extension(cert, oid, NULL, ext_value_len, NULL));
+
+    return S2N_SUCCESS;
+}
+
+int s2n_cert_get_x509_extension_value(struct s2n_cert *cert, const uint8_t *oid,
+                                      uint8_t *ext_value, uint32_t *ext_value_len, bool *critical)
+{
+    POSIX_ENSURE_REF(cert);
+    POSIX_ENSURE_REF(oid);
+    POSIX_ENSURE_REF(ext_value);
+    POSIX_ENSURE_REF(ext_value_len);
+    POSIX_ENSURE_REF(critical);
+
+    POSIX_GUARD(s2n_parse_x509_extension(cert, oid, ext_value, ext_value_len, critical));
+
+    return S2N_SUCCESS;
+}
