@@ -54,10 +54,11 @@ static int s2n_client_key_share_recv(struct s2n_connection *conn, struct s2n_stu
 
 const s2n_extension_type s2n_client_key_share_extension = {
     .iana_value = TLS_EXTENSION_KEY_SHARE,
+    .minimum_version = S2N_TLS13,
     .is_response = false,
     .send = s2n_client_key_share_send,
     .recv = s2n_client_key_share_recv,
-    .should_send = s2n_extension_send_if_tls13_connection,
+    .should_send = s2n_extension_always_send,
     .if_missing = s2n_extension_noop_if_missing,
 };
 
@@ -416,10 +417,6 @@ static int s2n_client_key_share_recv(struct s2n_connection *conn, struct s2n_stu
     POSIX_ENSURE_REF(conn);
     POSIX_ENSURE_REF(extension);
 
-    if (s2n_connection_get_protocol_version(conn) < S2N_TLS13) {
-        return S2N_SUCCESS;
-    }
-
     const struct s2n_ecc_preferences *ecc_pref = NULL;
     POSIX_GUARD(s2n_connection_get_ecc_preferences(conn, &ecc_pref));
     POSIX_ENSURE_REF(ecc_pref);
