@@ -27,10 +27,11 @@ static int s2n_server_key_share_recv(struct s2n_connection *conn, struct s2n_stu
 
 const s2n_extension_type s2n_server_key_share_extension = {
     .iana_value = TLS_EXTENSION_KEY_SHARE,
+    .minimum_version = S2N_TLS13,
     .is_response = false,
     .send = s2n_server_key_share_send,
     .recv = s2n_server_key_share_recv,
-    .should_send = s2n_extension_send_if_tls13_connection,
+    .should_send = s2n_extension_always_send,
     .if_missing = s2n_extension_noop_if_missing,
 };
 
@@ -286,10 +287,6 @@ static int s2n_server_key_share_recv_ecc(struct s2n_connection *conn, uint16_t n
  */
 static int s2n_server_key_share_recv(struct s2n_connection *conn, struct s2n_stuffer *extension)
 {
-    if (s2n_connection_get_protocol_version(conn) < S2N_TLS13) {
-        return S2N_SUCCESS;
-    }
-
     POSIX_ENSURE_REF(conn);
     POSIX_ENSURE_REF(extension);
 
