@@ -45,7 +45,7 @@
 #include "utils/s2n_safety.h"
 
 struct s2n_client_hello *s2n_connection_get_client_hello(struct s2n_connection *conn) {
-    if (conn->client_hello.parsed != 1) {
+    if (conn->client_hello.callback_invoked != 1) {
         return NULL;
     }
 
@@ -111,6 +111,20 @@ ssize_t s2n_client_hello_get_cipher_suites_length(struct s2n_client_hello *ch) {
     return ch->cipher_suites.size;
 }
 
+int s2n_client_hello_cb_done(struct s2n_connection *conn)
+{
+    POSIX_ENSURE_REF(conn);
+    POSIX_ENSURE_REF(conn->config);
+    POSIX_ENSURE(conn->config->client_hello_cb_mode ==
+        S2N_CLIENT_HELLO_CB_NONBLOCKING, S2N_ERR_INVALID_STATE);
+    POSIX_ENSURE(conn->client_hello.callback_invoked == 1, S2N_ERR_ASYNC_NOT_PERFORMED);
+
+    conn->client_hello.callback_async_blocked = 0;
+    conn->client_hello.callback_async_done = 1;
+
+    return S2N_SUCCESS;
+}
+
 ssize_t s2n_client_hello_get_cipher_suites(struct s2n_client_hello *ch, uint8_t *out, uint32_t max_length)
 {
     POSIX_ENSURE_REF(ch);
@@ -154,6 +168,11 @@ int s2n_client_hello_free(struct s2n_client_hello *client_hello)
     client_hello->cipher_suites.data = NULL;
     client_hello->extensions.raw.data = NULL;
 
+    /* clean the CH nonblocking callback flags
+     * incase we are preparing for CH retry */
+    client_hello->callback_async_blocked = 0;
+    client_hello->callback_async_done = 0;
+
     return 0;
 }
 
@@ -296,27 +315,52 @@ int s2n_process_client_hello(struct s2n_connection *conn)
     return S2N_SUCCESS;
 }
 
+static S2N_RESULT s2n_client_hello_process_cb_response(struct s2n_connection *conn, int rc)
+{
+    if (rc < 0) {
+        goto fail;
+    }
+    switch(conn->config->client_hello_cb_mode) {
+        case S2N_CLIENT_HELLO_CB_BLOCKING : {
+            if(rc) {
+                conn->server_name_used = 1;
+            }
+            return S2N_RESULT_OK;
+        }
+        case S2N_CLIENT_HELLO_CB_NONBLOCKING : {
+            if (conn->client_hello.callback_async_done) {
+                return S2N_RESULT_OK;
+            }
+            conn->client_hello.callback_async_blocked = 1;
+            RESULT_BAIL(S2N_ERR_ASYNC_BLOCKED);
+        }
+    }
+fail:
+    /* rc < 0 */
+    RESULT_GUARD_POSIX(s2n_queue_reader_handshake_failure_alert(conn));
+    RESULT_BAIL(S2N_ERR_CANCELLED);
+}
+
 int s2n_client_hello_recv(struct s2n_connection *conn)
 {
-    /* Parse client hello */
-    POSIX_GUARD(s2n_parse_client_hello(conn));
+    POSIX_ENSURE(conn->client_hello.callback_async_blocked == 0, S2N_ERR_ASYNC_BLOCKED);
 
+    if (conn->config->client_hello_cb_mode == S2N_CLIENT_HELLO_CB_BLOCKING ||
+            !conn->client_hello.callback_async_done)
+    {
+        /* Parse client hello */
+        POSIX_GUARD(s2n_parse_client_hello(conn));
+    }
     /* If the CLIENT_HELLO has already been parsed, then we should not call
      * the client_hello_cb a second time. */
-    if (conn->client_hello.parsed == 0) {
+    if (conn->client_hello.callback_invoked == 0) {
         /* Mark the collected client hello as available when parsing is done and before the client hello callback */
-        conn->client_hello.parsed = 1;
+        conn->client_hello.callback_invoked = 1;
 
         /* Call client_hello_cb if exists, letting application to modify s2n_connection or swap s2n_config */
         if (conn->config->client_hello_cb) {
             int rc = conn->config->client_hello_cb(conn, conn->config->client_hello_cb_ctx);
-            if (rc < 0) {
-                POSIX_GUARD(s2n_queue_reader_handshake_failure_alert(conn));
-                POSIX_BAIL(S2N_ERR_CANCELLED);
-            }
-            if (rc) {
-                conn->server_name_used = 1;
-            }
+            POSIX_GUARD_RESULT(s2n_client_hello_process_cb_response(conn, rc));
         }
     }
 
