@@ -20,15 +20,7 @@
 #include "stuffer/s2n_stuffer.h"
 
 #define S2N_STATE_LIFETIME_IN_NANOS     54000000000000      /* 15 hours */
-#define S2N_STATE_SIZE_IN_BYTES         (1 + 8 + 1 + S2N_TLS_CIPHER_SUITE_LEN + S2N_TLS_SECRET_LEN)
-
-#define S2N_MAX_STATE_SIZE_IN_BYTES     sizeof(uint8_t)  +  /* serialization format */  \
-                                        sizeof(uint8_t)  +  /* protocol version */      \
-                                        sizeof(uint16_t) +  /* cipher suite */          \
-                                        sizeof(uint64_t) +  /* ticket issue time */     \
-                                        sizeof(uint32_t) +  /* ticket age add */        \
-                                        sizeof(uint8_t)  +  /* secret size */           \
-                                        S2N_TLS_SECRET_LEN
+#define S2N_TLS12_STATE_SIZE_IN_BYTES   (1 + 8 + 1 + S2N_TLS_CIPHER_SUITE_LEN + S2N_TLS_SECRET_LEN)
 
 #define S2N_TLS_SESSION_CACHE_TTL       (6 * 60 * 60)
 #define S2N_TICKET_KEY_NAME_LEN         16
@@ -36,12 +28,10 @@
 #define S2N_TICKET_AAD_LEN              (S2N_TICKET_AAD_IMPLICIT_LEN + S2N_TICKET_KEY_NAME_LEN)
 #define S2N_AES256_KEY_LEN              32
 #define ONE_SEC_IN_NANOS                1000000000
+#define ONE_MILLISEC_IN_NANOS           1000000
 #define ONE_WEEK_IN_SEC                 604800
 #define S2N_TLS12_TICKET_SIZE_IN_BYTES  (S2N_TICKET_KEY_NAME_LEN + S2N_TLS_GCM_IV_LEN +     \
-        S2N_STATE_SIZE_IN_BYTES + S2N_TLS_GCM_TAG_LEN)
-
-#define S2N_MAX_TICKET_SIZE_IN_BYTES    (S2N_TICKET_KEY_NAME_LEN + S2N_TLS_GCM_IV_LEN +     \
-        S2N_MAX_STATE_SIZE_IN_BYTES + S2N_TLS_GCM_TAG_LEN)
+        S2N_TLS12_STATE_SIZE_IN_BYTES + S2N_TLS_GCM_TAG_LEN)
 
 #define S2N_TICKET_ENCRYPT_DECRYPT_KEY_LIFETIME_IN_NANOS        7200000000000     /* 2 hours */
 #define S2N_TICKET_DECRYPT_KEY_LIFETIME_IN_NANOS                46800000000000    /* 13 hours */
@@ -51,6 +41,11 @@
 #define S2N_GREATER_OR_EQUAL            1
 #define S2N_LESS_THAN                  -1
 
+#define S2N_TLS12_SESSION_SIZE          S2N_STATE_FORMAT_LEN + \
+                                        S2N_SESSION_TICKET_SIZE_LEN + \
+                                        S2N_TLS12_TICKET_SIZE_IN_BYTES + \
+                                        S2N_TLS12_STATE_SIZE_IN_BYTES
+
 struct s2n_connection;
 struct s2n_config;
 
@@ -71,6 +66,11 @@ struct s2n_ticket_fields {
     uint32_t ticket_age_add;
 };
 
+struct s2n_session_ticket {
+    struct s2n_blob ticket_data;
+    uint32_t session_lifetime;
+};
+
 extern struct s2n_ticket_key *s2n_find_ticket_key(struct s2n_config *config, const uint8_t *name);
 extern int s2n_encrypt_session_ticket(struct s2n_connection *conn, struct s2n_ticket_fields *ticket_fields, struct s2n_stuffer *to);
 extern int s2n_decrypt_session_ticket(struct s2n_connection *conn);
@@ -94,8 +94,16 @@ typedef enum {
 extern int s2n_allowed_to_cache_connection(struct s2n_connection *conn);
 extern int s2n_resume_from_cache(struct s2n_connection *conn);
 extern int s2n_store_to_cache(struct s2n_connection *conn);
+int s2n_client_serialize_resumption_state(struct s2n_connection *conn, struct s2n_ticket_fields *ticket_fields, struct s2n_stuffer *to);
 
 /* These functions will be labeled S2N_API and become a publicly visible api 
  * once we release the session resumption API. */
 int s2n_config_set_initial_ticket_count(struct s2n_config *config, uint8_t num);
 int s2n_connection_add_new_tickets_to_send(struct s2n_connection *conn, uint8_t num);
+
+struct s2n_session_ticket;
+typedef int (*s2n_session_ticket_fn)(struct s2n_connection *conn, struct s2n_session_ticket *ticket);
+int s2n_config_set_session_ticket_cb(struct s2n_config *config, s2n_session_ticket_fn callback, void *ctx);
+int s2n_session_ticket_get_data_len(struct s2n_session_ticket *ticket, size_t *data_len);
+int s2n_session_ticket_get_data(struct s2n_session_ticket *ticket, size_t max_data_len, uint8_t *data);
+int s2n_session_ticket_get_lifetime(struct s2n_session_ticket *ticket, uint32_t *session_lifetime);
